/**
 * Copyright (C) 2007-2012 Linutop SAS <http://www.linutop.com/>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#ifdef HAVE_NET_IF_H
#include <net/if.h>
#endif
#ifdef HAVE_SYS_IOCTL_H
#include <sys/ioctl.h>
#endif
#ifdef HAVE_SYS_STATFS_H
#include <sys/statfs.h>
#endif
#ifdef HAVE_MNTENT_H
#include <mntent.h>
#endif

#include <ltop_os_info.h>

#include "main.h"
#include "libxfcegui4.h"

#define MNT "/proc/mounts"

const gchar *persist_char[2] = {
    N_("Writable mode"),
    N_("Locked mode")
};

size_t
get_mem_total ()
{
    FILE *f;
    char line[256];
    size_t memtotal = 0;

    if ((f = fopen ("/proc/meminfo", "r")) == NULL)
        return 0;

    while (fgets (line, 255, f))
    {
        char name[64];
        size_t l;
        int ret;
        ret = sscanf (line, "%s %ld", name, &l);
        if ((ret == 2) && (strcmp (name, "MemTotal:") == 0))
        {
            memtotal = l;
            break;
        }
    }

    fclose (f);

    return memtotal * 1000;
}

static void
get_ifaces (GHashTable *hash)
{
    char buf[1024];
    struct ifconf ifc;
    struct ifreq *ifr;
    int fd;
    int i, ni;

    fd = socket(AF_INET,SOCK_DGRAM, 0);
    ifc.ifc_len = sizeof(buf);
    ifc.ifc_buf = buf;
    ioctl (fd, SIOCGIFCONF, &ifc);
    ifr = ifc.ifc_req;
    ni = ifc.ifc_len / sizeof (struct ifreq);

    for (i=0;i<ni;i++)
    {
        struct ifreq *item = &ifr[i];
        if (ioctl(fd, SIOCGIFADDR, item) == 0)
        {
            /* forget about lo */
            if (!strcmp (item->ifr_name, "lo"))
                continue;

            gchar ip[16];
            g_sprintf (ip, "%d.%d.%d.%d",
                (unsigned char) item->ifr_addr.sa_data[2],
                (unsigned char) item->ifr_addr.sa_data[3],
                (unsigned char) item->ifr_addr.sa_data[4],
                (unsigned char) item->ifr_addr.sa_data[5]);
            g_hash_table_insert (hash, g_strdup (item->ifr_name), g_strdup (ip));
        }
    }
}

static void
str_from_hash (gpointer key, gpointer value, gchar ** data)
{
    gchar *net_devs = NULL;
    gchar *tmp = *data;
    net_devs = g_strconcat (tmp ? tmp : "",
                            (gchar *) key, " => ",
                            (gchar *) value,"\n",
                            NULL);
    g_free (tmp);
    *data = net_devs;
}

static void
get_freespace (gchar **freestr)
{
    struct statfs state;
    guint64 freespace = 0;

    if (!statfs ("/cdrom/boot", &state))
        freespace = (guint64) state.f_bfree * (guint64) state.f_bsize;
    if (freespace > 0)
        *freestr = g_format_size (freespace);
    else if (freespace < 0)
        *freestr = g_strdup_printf (_("Unable to determine"));
    else
        *freestr = g_strdup (_("No space left"));
}

GtkWidget *
get_info_widget ()
{
    GtkWidget *table, *vbox, *hbox, *label;
    GHashTable *hash;
    gchar *total_ram;
    gchar *net_devs = NULL;
    gchar *freespace;
    LtopOSInfo *info;

    hash = g_hash_table_new_full (g_str_hash, g_str_equal,
                                  g_free, g_free);
    get_ifaces (hash);
    info = ltop_os_info_get ();
    get_freespace (&freespace);

    total_ram = g_format_size ((guint64) (get_mem_total ()));

    vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);

    label = gtk_label_new (NULL);
    gtk_label_set_markup (GTK_LABEL (label),
        _("<span weight=\"bold\" size=\"large\" style=\"italic\">"
          "System Configuration</span>"));
    gtk_misc_set_alignment (GTK_MISC (label), 0.5, 0);
    gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, FALSE, 8);

    table = gtk_table_new (3, 4, FALSE);
    gtk_table_set_col_spacing (GTK_TABLE (table), 1, 24);
    hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
    gtk_box_pack_start (GTK_BOX (hbox), table, TRUE, FALSE, 0);

    label = gtk_label_new (NULL);
    gtk_label_set_markup (GTK_LABEL (label),
        _("<span weight=\"bold\">Root device: </span>"));
    gtk_table_attach (GTK_TABLE (table), label, 2, 3, 0, 1,
                      0, GTK_FILL, 0, 0);
    gtk_misc_set_alignment (GTK_MISC (label), 0, 0);

    label = gtk_label_new (info->root_device);
    gtk_table_attach (GTK_TABLE (table), label, 3, 4, 0, 1,
                      GTK_FILL | GTK_EXPAND, GTK_FILL, 0, 0);
    gtk_misc_set_alignment (GTK_MISC (label), 0, 0);

    label = gtk_label_new (NULL);
    gtk_label_set_markup (GTK_LABEL (label),
        _("<span weight=\"bold\">Network: </span>"));
    gtk_table_attach (GTK_TABLE (table), label,
                      0, 1,
#ifdef WITH_DEMO
                      0, 1,
#else
                      2, 3,
#endif
                      0, GTK_FILL, 0, 0);
    gtk_misc_set_alignment (GTK_MISC (label), 0, 0);
    g_hash_table_foreach (hash, (GHFunc) str_from_hash, &net_devs);
    if (net_devs)
        net_devs[g_utf8_strlen (net_devs, -1) - 1] = '\0';
    label = gtk_label_new (net_devs);
    g_hash_table_destroy (hash);
    g_free (net_devs);

    gtk_table_attach (GTK_TABLE (table), label,
                      1, 2,
#ifdef WITH_DEMO
                      0, 1,
#else
                      2, 3,
#endif
                      GTK_FILL | GTK_EXPAND, GTK_FILL, 0, 0);
    gtk_misc_set_alignment (GTK_MISC (label), 0, 0);

#ifndef WITH_DEMO
    label = gtk_label_new (NULL);
    gtk_label_set_markup (GTK_LABEL (label),
        _("<span weight=\"bold\">Free disk space: </span>"));
    gtk_table_attach (GTK_TABLE (table), label, 0, 1, 0, 1,
                      0, GTK_FILL, 0, 0);
    gtk_misc_set_alignment (GTK_MISC (label), 0, 0);

    label = gtk_label_new (freespace);
    gtk_table_attach (GTK_TABLE (table), label, 1, 2, 0, 1,
                      GTK_FILL | GTK_EXPAND, GTK_FILL, 0, 0);
    gtk_misc_set_alignment (GTK_MISC (label), 0, 0);

    label = gtk_label_new (NULL);
    gtk_label_set_markup (GTK_LABEL (label),
        _("<span weight=\"bold\">Security: </span>"));
    gtk_table_attach (GTK_TABLE (table), label, 2, 3, 1, 2,
                      0, GTK_FILL, 0, 0);
    gtk_misc_set_alignment (GTK_MISC (label), 0, 0);

    label = gtk_label_new (_(persist_char[info->locked]));
    gtk_table_attach (GTK_TABLE (table), label, 3, 4, 1, 2,
                      GTK_FILL | GTK_EXPAND, GTK_FILL, 0, 0);
    gtk_misc_set_alignment (GTK_MISC (label), 0, 0);

    label = gtk_label_new (NULL);
    gtk_label_set_markup (GTK_LABEL (label),
        _("<span weight=\"bold\">Total RAM: </span>"));
    gtk_table_attach (GTK_TABLE (table), label, 0, 1, 1, 2,
                      0, GTK_FILL, 0, 0);
    gtk_misc_set_alignment (GTK_MISC (label), 0, 0);

    label = gtk_label_new (total_ram);
    gtk_table_attach (GTK_TABLE (table), label, 1, 2, 1, 2,
                      GTK_FILL | GTK_EXPAND, GTK_FILL, 0, 0);
    gtk_misc_set_alignment (GTK_MISC (label), 0, 0);

    g_free (total_ram);
#endif /* WITH_DEMO */

    gtk_box_pack_start (GTK_BOX (vbox), hbox, TRUE, TRUE, 2);
    gtk_widget_show_all (vbox);

    return vbox;
}

GtkWidget *
get_about_widget ()
{
    GtkWidget *box, *hs, *hbox;
    GtkWidget *label;

    box = gtk_box_new (GTK_ORIENTATION_VERTICAL, 2);

    hs = gtk_separator_new (GTK_ORIENTATION_HORIZONTAL);
    gtk_box_pack_start (GTK_BOX (box), hs, FALSE, FALSE, 8);

    label = gtk_label_new (NULL);
    gtk_label_set_markup (GTK_LABEL (label),
        _("<span size=\"large\" weight=\"bold\" style=\"italic\">"
        "Linutop Configuration Panel V. " PACKAGE_VERSION "</span>"));
    gtk_box_pack_start (GTK_BOX (box), label, FALSE, FALSE, 2);

    label = gtk_label_new (NULL);
    gtk_label_set_markup (GTK_LABEL (label),
        _("Designed for quick and simple setup of your Linutop system."));
    gtk_misc_set_alignment (GTK_MISC (label), 0.5, 0);
    gtk_misc_set_padding (GTK_MISC (label), 6, 0);
    gtk_box_pack_start (GTK_BOX (box), label, FALSE, FALSE, 6);

    hs = gtk_separator_new (GTK_ORIENTATION_HORIZONTAL);
    gtk_box_pack_start (GTK_BOX (box), hs, FALSE, FALSE, 8);

    label = gtk_label_new (NULL);
    gtk_label_set_markup (GTK_LABEL (label),
        _("<span size=\"large\" weight=\"bold\" style=\"italic\">"
          "Linutop: Energy Efficient Computing</span>"));
    gtk_misc_set_alignment (GTK_MISC (label), 0.5, 0);
    gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_CENTER);
    gtk_box_pack_start (GTK_BOX (box), label, FALSE, FALSE, 6);

    label = gtk_label_new (NULL);
    gtk_label_set_markup (GTK_LABEL (label),
        _("Maintenance free system for: Public internet access, "
          "Digital signage,\nTelecom, and many other uses."));
    gtk_misc_set_alignment (GTK_MISC (label), 0.5, 0);
    gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_CENTER);
    gtk_box_pack_start (GTK_BOX (box), label, FALSE, FALSE, 6);

    hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
    gtk_box_pack_start (GTK_BOX (box), hbox, FALSE, FALSE, 6);

    label = gtk_label_new (NULL);
    gtk_label_set_markup (GTK_LABEL (label),
        _("More on <a href=\"http://doc.linutop.com/\">Linutop Documentation</a>"));
    gtk_misc_set_alignment (GTK_MISC (label), 0.5, 0);
    gtk_misc_set_padding (GTK_MISC (label), 6, 0);
    gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 6);

    label = gtk_label_new (NULL);
    gtk_label_set_markup (GTK_LABEL (label),
        _("Thank you for using Linutop!"));
    gtk_misc_set_alignment (GTK_MISC (label), 0.5, 0);
    gtk_misc_set_padding (GTK_MISC (label), 6, 0);
    gtk_box_pack_end (GTK_BOX (hbox), label, FALSE, FALSE, 6);

    gtk_widget_show_all (box);

    return box;
}
