/* language.c
 * Copyright (C) 2007-2012 Linutop SAS <http://www.linutop.com/>
 * This file is part of the linutop config editor
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 2 of the license.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "main.h"
#include "utils.h"

static GtkWidget *progress_dialog;
static volatile int done = 0;

GtkTreeIter
populate_locale_tree (GtkTreeStore * treestore)
{
    int i = 0;
    gboolean found = FALSE;
    GtkTreeIter iter;
    GtkTreeIter ret;
    gchar *code = g_strdup (get_locale_value ());

    while (locale_data[i].code != NULL) {
        gtk_tree_store_append (treestore, &iter, NULL);
        gtk_tree_store_set (treestore, &iter,
                   0, locale_data[i].name,
                   1, locale_data[i].code,
                   2, locale_data[i].pkgs,
                   -1);
        if (strcmp (locale_data[i].code, code) == 0) {
            ret = iter;
            found = TRUE;
        }
        i++;
    }
    g_free (code);

    if (!found)
        gtk_tree_model_get_iter_first (GTK_TREE_MODEL (treestore), &ret);
    return ret;
}

static gpointer
ui_thread (gpointer data)
{
    progress_dialog = gtk_message_dialog_new (NULL,
                                     GTK_DIALOG_MODAL,
                                     GTK_MESSAGE_INFO,
                                     GTK_BUTTONS_NONE,
                                     _("Please wait while installing..."));
    gtk_window_set_deletable (GTK_WINDOW (progress_dialog), FALSE);
    gtk_widget_show (progress_dialog);

    return NULL;
}

static gpointer
install_thread (gpointer data)
{
    gchar *cmd;

    g_spawn_command_line_sync ("apt-get update", NULL, NULL, NULL, NULL);
    cmd = g_strdup_printf ("apt-get install -y --force-yes %s", (gchar *)data);
    g_spawn_command_line_sync (cmd, NULL, NULL, NULL, NULL);
    g_free (cmd);
    g_atomic_int_set (&done, 1);
    return NULL;
}

static gboolean
check_done (gpointer data)
{
    if (g_atomic_int_get (&done) == 1)
    {
        gtk_widget_destroy (progress_dialog);
        g_atomic_int_set (&done, 0);
        return FALSE;
    }

    return TRUE;
}

static void
run_install_dialog (const gchar *pkgs)
{
    GtkWidget *d;
    int response;
    GThread *ui_t;

    d = gtk_message_dialog_new (NULL,
                                GTK_DIALOG_MODAL,
                                GTK_MESSAGE_QUESTION,
                                GTK_BUTTONS_YES_NO,
                                _("Some packages need to be installed to enable this language.\n"
                                  "Should they be installed now?"));
    response = gtk_dialog_run (GTK_DIALOG (d));
    gtk_widget_destroy (d);

    if (response == GTK_RESPONSE_YES)
    {
        ui_t = g_thread_new ("ui", ui_thread, NULL);
        g_thread_join (ui_t);
        g_thread_new ("install", install_thread, (gpointer) pkgs);

        g_timeout_add (1, check_done, NULL);
    }
}

static void
maybe_install_pkgs (const gchar *pkgs)
{
    char **list;
    char **p;
    char *tmp = NULL;
    gboolean needs_install = FALSE;

    list = g_strsplit (pkgs, " ", -1);
    for (p=list; *p; p++)
    {
        tmp = g_strdup_printf ("/var/lib/dpkg/info/%s.list", *p);
        if (g_access (tmp, R_OK) != 0)
            needs_install = TRUE;

        g_free (tmp);
    }
    g_strfreev (list);

    if (needs_install)
        run_install_dialog (pkgs);
}

/* Dialog code */
void
locale_dialog_run ()
{
    GtkWidget *new_dialog, *scw1, *hbox;
    GtkWidget *tree_view = gtk_tree_view_new ();
    GtkTreeStore *treestore;
    GtkCellRenderer *renderer;
    GtkTreeSelection *selection;
    GtkTreeIter iter, default_iter;
    GtkTreePath *default_iter_path;
    GtkTreeModel *model;

    new_dialog =
        xfce_titled_dialog_new_with_buttons (_("Linutop - Language"),
                        GTK_WINDOW (dialog->dialog),
                        GTK_DIALOG_MODAL |
                        GTK_DIALOG_DESTROY_WITH_PARENT,
                        GTK_STOCK_CANCEL,
                        GTK_RESPONSE_CANCEL,
                        GTK_STOCK_OK, GTK_RESPONSE_OK,
                        NULL);
    gtk_window_set_icon_name (GTK_WINDOW (new_dialog),
                 "preferences-desktop-locale");
    xfce_titled_dialog_set_subtitle ((XfceTitledDialog *) new_dialog,
                    _("Select a language"));

    hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 6);
    gtk_box_set_homogeneous (GTK_BOX (hbox), TRUE);
    gtk_box_pack_start (GTK_BOX (
            gtk_dialog_get_content_area (GTK_DIALOG (new_dialog))),
            hbox, TRUE, TRUE, 0);

    scw1 = gtk_scrolled_window_new (NULL, NULL);
    gtk_box_pack_start (GTK_BOX (hbox), scw1, TRUE, TRUE, 2);
    gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scw1),
                       GTK_POLICY_AUTOMATIC,
                       GTK_POLICY_AUTOMATIC);

    /* tree */
    treestore = gtk_tree_store_new (4, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING);
    default_iter = populate_locale_tree (GTK_TREE_STORE (treestore));
    renderer = gtk_cell_renderer_text_new ();

    gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (tree_view),
                            -1,
                            "Language",
                            renderer, "text", 0, NULL);
    gtk_tree_view_set_model (GTK_TREE_VIEW (tree_view),
                GTK_TREE_MODEL (treestore));
    gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (tree_view), FALSE);
    selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (tree_view));

    /* select default iter */
    gtk_tree_selection_select_iter (selection, &default_iter);

    /* scroll to the default iter */
    default_iter_path =
        gtk_tree_model_get_path (gtk_tree_view_get_model
                    (GTK_TREE_VIEW (tree_view)), &default_iter);
    gtk_tree_view_scroll_to_cell (GTK_TREE_VIEW (tree_view),
                     default_iter_path, NULL, TRUE, 0.5, 0.5);

    gtk_tree_path_free (default_iter_path);

    gtk_container_add (GTK_CONTAINER (scw1), tree_view);

    gtk_widget_show_all (hbox);

    gtk_window_set_default_size (GTK_WINDOW(new_dialog), 400, 400);

    g_signal_connect_swapped (G_OBJECT (tree_view), "row-activated",
             G_CALLBACK (cb_row_activated), new_dialog);

    int response = gtk_dialog_run (GTK_DIALOG (new_dialog));

    if (response == GTK_RESPONSE_OK) {
        gchar *locale;
        gchar *pkgs;

        selection =
            gtk_tree_view_get_selection (GTK_TREE_VIEW (tree_view));
        gtk_tree_selection_get_selected (selection, &model, &iter);
        gtk_tree_model_get (model, &iter, 1, &locale, 2, &pkgs, -1);
        maybe_install_pkgs (pkgs);
        set_locale (locale);
        update_locale_settings (locale);
        g_free (locale);
    }
    gtk_widget_destroy (new_dialog);
    return;
}
